﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using MugenInjection.Activators;
using MugenInjection;

namespace NServiceBus.ObjectBuilder.MugenInjection
{
    /// <summary>
    /// Represents the activator for <see cref="MugenInjectionObjectBuilder"/>.
    /// </summary>
    public class ObjectBuilderActivator : ExpressionActivator
    {
        #region Fields

        private Type _serviceType;
        private List<PropertyInfo> _cachedProperty;

        #endregion

        #region Method

        private IEnumerable<PropertyInfo> GetProperties(Type service)
        {
            if (_serviceType != service)
            {
                _serviceType = service;
                _cachedProperty = service.GetProperties(BindingFlags).ToList();
                return _cachedProperty;
            }
            return _cachedProperty;
        }

        #endregion

        #region Overrides of ActivatorBase

        /// <summary>
        /// Indicates that the service has values to inject.
        /// </summary>
        /// <param name="service">The specified service.</param>
        /// <param name="attributeType">The specified attribute type that use for inject.</param>
        /// <returns>If <c>true</c> service have values for inject; otherwise <c>false</c>.</returns>
        protected override bool IsHasValuesToInject(Type service, Type attributeType)
        {
            return true;
        }

        /// <summary>
        ///     Gets a series of properties to inject.
        /// </summary>
        /// <param name="service">The specified service type.</param>
        /// <param name="attributeType">The specified attribute to inject.</param>
        /// <returns>A series of properties tp inject.</returns>
        protected override IList<PropertyInfo> GetPropertiesToInject(Type service, Type attributeType)
        {
            IList<PropertyInfo> propertiesToInject = base.GetPropertiesToInject(service, attributeType);
            foreach (PropertyInfo propertyInfo in GetProperties(service))
            {
                if (propertiesToInject.Contains(propertyInfo)) continue;
                if (CurrentContext.Parameters.Any(parameter => parameter.CanResolve(propertyInfo, null)))
                {
                    propertiesToInject.Add(propertyInfo);
                    continue;
                }
                if (!CurrentContext.CallInjector.CanResolve(propertyInfo.PropertyType, true, false)) continue;
                propertiesToInject.Add(propertyInfo);
            }
            return propertiesToInject;
        }

        #endregion
    }
}