﻿using System;
using System.Data.Services;

namespace MugenInjection.Wcf.DataServiceHostImpl
{
    /// <summary>
    /// A web service host that uses MugenInjection to create the service instances.
    /// </summary>
    public class MugenInjectionDataServiceHost : DataServiceHost
    {
        #region Constructor

        /// <summary>
        /// Instantiates <see cref="MugenInjectionDataServiceHost"/> for an ADO.NET Data Service.
        /// </summary>
        /// <param name="serviceType">Identifies the ADO.NET Data Service to the host.</param><param name="baseAddresses">The URI of the host.</param>
        public MugenInjectionDataServiceHost(Type serviceType, Uri[] baseAddresses)
            : base(serviceType, baseAddresses)
        {
        }

        #endregion

        #region Overrides of ServiceHostBase

        /// <summary>
        /// Called when the <see cref="T:System.ServiceModel.Web.WebServiceHost"/> instance opens.
        /// </summary>
        protected override void OnOpening()
        {
            WcfConfigExtension.InitializeServiceHostOnOpening(this);
            base.OnOpening();
        }

        /// <summary>
        /// Initializes the runtime for the service host.
        /// </summary>
        /// <exception cref="T:System.InvalidOperationException">The description of the service hosted is null.</exception>
        protected override void InitializeRuntime()
        {
            base.InitializeRuntime();
            WcfConfigExtension.InitializeServiceHostOnInitializeRuntime(this);
        }

        #endregion
    }
}