﻿using System;
using System.ServiceModel;

namespace MugenInjection.Wcf.ServiceHostImpl
{
    /// <summary>
    /// A service host that uses MugenInjection to create the service instances.
    /// </summary>
    public abstract class MugenInjectionServiceHostBase : ServiceHost
    {
        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="MugenInjectionServiceHostBase"/> class. 
        /// </summary>
        protected MugenInjectionServiceHostBase()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MugenInjectionServiceHostBase"/> class with the type of service and its base addresses specified.
        /// </summary>
        /// <param name="serviceType">The type of hosted service.</param><param name="baseAddresses">An array of type <see cref="T:System.Uri"/> that contains the base addresses for the hosted service.</param><exception cref="T:System.ArgumentNullException"><paramref name="serviceType"/> is null.</exception>
        protected MugenInjectionServiceHostBase(Type serviceType, params Uri[] baseAddresses)
            : base(serviceType, baseAddresses)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MugenInjectionServiceHostBase"/> class with the instance of the service and its base addresses specified.
        /// </summary>
        /// <param name="singletonInstance">The instance of the hosted service.</param><param name="baseAddresses">An <see cref="T:System.Array"/> of type <see cref="T:System.Uri"/> that contains the base addresses for the hosted service.</param><exception cref="T:System.ArgumentNullException"><paramref name="singletonInstance"/> is null.</exception>
        protected MugenInjectionServiceHostBase(object singletonInstance, params Uri[] baseAddresses)
            : base(singletonInstance, baseAddresses)
        {
        }

        #endregion

        #region Overrides of ServiceHostBase

        /// <summary>
        /// Called when the <see cref="T:System.ServiceModel.Web.WebServiceHost"/> instance opens.
        /// </summary>
        protected override void OnOpening()
        {
            WcfConfigExtension.InitializeServiceHostOnOpening(this);
            base.OnOpening();
        }

        /// <summary>
        /// Initializes the runtime for the service host.
        /// </summary>
        /// <exception cref="T:System.InvalidOperationException">The description of the service hosted is null.</exception>
        protected override void InitializeRuntime()
        {
            base.InitializeRuntime();
            WcfConfigExtension.InitializeServiceHostOnInitializeRuntime(this);
        }

        #endregion
    }
}