﻿using System;
using System.Reflection;
using MugenInjection.Delegates;
using MugenInjection.Infrastructure;

namespace MugenInjection.Parameters
{
    /// <summary>
    ///     Represents the value of a property parameter.
    /// </summary>
    public sealed class PropertyParameter : InjectionParameter
    {
        #region Constructors

        /// <summary>
        ///     Initializes a new instance of the <see cref="PropertyParameter" /> class.
        /// </summary>
        public PropertyParameter(string name, ParameterDelegate<object> getValueDelegate, Type type = null)
            : base(MemberTypes.Property, name, getValueDelegate)
        {
            // ReSharper disable DoNotCallOverridableMethodsInConstructor
            ParameterType = type;
            // ReSharper restore DoNotCallOverridableMethodsInConstructor
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="PropertyParameter" /> class.
        /// </summary>
        public PropertyParameter(string name, object value)
            : base(MemberTypes.Property, name, value)
        {
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="PropertyParameter" /> class.
        /// </summary>
        public PropertyParameter(string name, Type parameterType, object value)
            : base(MemberTypes.Property, name, value)
        {
            Validate.ArgumentNotNull(parameterType, "parameterType");
            // ReSharper disable DoNotCallOverridableMethodsInConstructor
            ParameterType = parameterType;
            // ReSharper restore DoNotCallOverridableMethodsInConstructor
        }

        #endregion

        #region Overrides of InjectionParameterBase

        /// <summary>
        ///     Determines whether the specified request can be resolved.
        /// </summary>
        /// <param name="memberInfo">
        ///     The specified <see cref="MemberInfo" />
        /// </param>
        /// <param name="parameter">
        ///     The specified <see cref="ParameterInfo" />
        /// </param>
        /// <returns>
        ///     <c>True</c> if the specified request has been resolved; otherwise, <c>false</c>.
        /// </returns>
        protected override bool CanResolveInternal(MemberInfo memberInfo, ParameterInfo parameter)
        {
            var propertyInfo = (PropertyInfo)memberInfo;
            return Name.Equals(memberInfo.Name) && IsAssignableParameter(propertyInfo.PropertyType);
        }

        #endregion
    }
}