﻿using System;
using System.Collections.Generic;
using MugenInjection.Core;
using MugenInjection.Delegates;
using MugenInjection.Infrastructure;
using MugenInjection.Interface;

namespace MugenInjection.Scope
{
    /// <summary>
    ///     Represents the custom lifecycle.
    /// </summary>
    internal sealed class CustomScopeLifecycle : DisposableObject, IScopeLifecycle
    {
        #region Fields

        private CustomScopeDelegate _customScopeDelegate;
        private IScopeLifecycleContext _context;

        #endregion

        #region Constructor

        /// <summary>
        ///     Initializes a new instance of the <see cref="CustomScopeLifecycle" /> class.
        /// </summary>
        /// <param name="customScopeDelegate"></param>
        public CustomScopeLifecycle(CustomScopeDelegate customScopeDelegate)
        {
            Validate.ArgumentNotNull(customScopeDelegate, "customScopeDelegate");
            _customScopeDelegate = customScopeDelegate;
        }

        #endregion

        #region Implementation of IScopeLifecycle

        /// <summary>
        ///     Gets or sets the <see cref="IScopeLifecycleContext" />.
        /// </summary>
        public IScopeLifecycleContext Context
        {
            get
            {
                Validate.EnsureIsInitializedValue(_context, "Context");
                return _context;
            }
            set { _context = value; }
        }

        /// <summary>
        ///     Gets the actions that should be called before instances are deactivated via the binding.
        /// </summary>
        ICollection<Action<object, bool>> IScopeLifecycle.ReleaseObjectActions
        {
            get { return null; }
        }

        /// <summary>
        ///     Resolves instance for the specified <see cref="IBindingContext" /> from scope.
        /// </summary>
        /// <returns>An instance of the service.</returns>
        public object GetObjectFromScope()
        {
            return _customScopeDelegate(Context);
        }

        /// <summary>
        ///     Releases all objects in current scope.
        /// </summary>
        /// <param name="isNeedDisposeObject">Indicating that need trying dispose objects.</param>
        public void ReleaseObjects(bool isNeedDisposeObject)
        {
        }

        #endregion

        #region Overrides of DisposableObject

        /// <summary>
        /// Releases resources held by the object.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing && !IsDisposed)
                _customScopeDelegate = null;
            base.Dispose(disposing);
        }

        #endregion
    }
}