﻿using System;
using MugenInjection.Delegates;
using MugenInjection.Interface;

namespace MugenInjection.Syntax.Method
{
    /// <summary>
    ///     Specific fluent syntax for method binding.
    /// </summary>
    public interface IMethodCallbackSyntax : IBindingBuilder
    {
        /// <summary>
        ///     Indicates that the binding should executes callback method before binding activated.
        /// </summary>
        /// <param name="onActivating">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax OnActivating(Action<IBindingContext> onActivating);

        /// <summary>
        ///     Indicates that the binding should executes callback method after binding activated.
        /// </summary>
        /// <param name="onActivated">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax OnActivated(Action<object> onActivated);

        /// <summary>
        ///     Indicates that the binding should executes callback method after binding activated.
        /// </summary>
        /// <param name="onActivated">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax OnActivated(BindingActivatedDelegate<object> onActivated);

        /// <summary>
        ///     Indicates that the binding should executes the callback that should be called before instances will be released.
        /// </summary>
        /// <param name="onReleaseObject">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax OnReleaseObject(Action<object, bool> onReleaseObject);

        /// <summary>
        ///     Indicates that the binding should executes callback method after binding disposed.
        /// </summary>
        /// <param name="onDisposed">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax OnDisposed(Action<IBinding> onDisposed);
    }

    /// <summary>
    ///     Specific fluent syntax for method binding.
    /// </summary>
    public interface IMethodCallbackSyntax<T> : IBindingBuilder
    {
        /// <summary>
        ///     Indicates that the binding should executes callback method before binding activated.
        /// </summary>
        /// <param name="onActivating">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> OnActivating(Action<IBindingContext> onActivating);

        /// <summary>
        ///     Indicates that the binding should executes callback method after binding activated.
        /// </summary>
        /// <param name="onActivated">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> OnActivated(Action<T> onActivated);

        /// <summary>
        ///     Indicates that the binding should executes callback method after binding activated.
        /// </summary>
        /// <param name="onActivated">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> OnActivated(BindingActivatedDelegate<T> onActivated);

        /// <summary>
        ///     Indicates that the binding should executes the callback that should be called before instances will be released.
        /// </summary>
        /// <param name="onReleaseObject">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> OnReleaseObject(Action<T, bool> onReleaseObject);

        /// <summary>
        ///     Indicates that the binding should executes callback method after binding disposed.
        /// </summary>
        /// <param name="onDisposed">The specified callback method.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> OnDisposed(Action<IBinding> onDisposed);
    }
}