﻿using MugenInjection.Infrastructure;
using MugenInjection.Interface;

namespace MugenInjection.Syntax.Method
{
    /// <summary>
    /// Specific fluent syntax for method binding.
    /// </summary>
    public interface IMethodPrioritySyntax : IBindingBuilder
    {
        /// <summary>
        ///  Sets the priority for binding.
        /// </summary>
        /// <param name="priority">The specified <see cref="BindingPriority"/>.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax SetPriority(BindingPriority priority);

        /// <summary>
        /// Sets a standard priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax SetStandardPriority();

        /// <summary>
        /// Sets a low priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax SetLowPriority();

        /// <summary>
        /// Sets a normal priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax SetNormalPriority();

        /// <summary>
        /// Sets a high priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax SetHighPriority();
    }

    /// <summary>
    /// Specific fluent syntax for method binding.
    /// </summary>
    public interface IMethodPrioritySyntax<T> : IBindingBuilder
    {
        /// <summary>
        /// Sets the priority for binding.
        /// </summary>
        /// <param name="priority">The specified <see cref="BindingPriority"/>.</param>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> SetPriority(BindingPriority priority);

        /// <summary>
        /// Sets a standard priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> SetStandardPriority();

        /// <summary>
        /// Sets a low priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> SetLowPriority();

        /// <summary>
        /// Sets a normal priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> SetNormalPriority();

        /// <summary>
        /// Sets a high priority for binding.
        /// </summary>
        /// <returns>The fluent syntax.</returns>
        IMethodCallbackObjectUseWithSyntax<T> SetHighPriority();
    }
}