﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MugenInjection.Infrastructure;
using MugenInjection.Interface;
using MugenInjection.Web;
using MugenInjection.Web.Interface;
using MugenInjection;
using MugenInjectionTest;
using MugenInjectionTest.Core;
using Should;

namespace MugenInjectionTest.Web.Mvc
{
    [TestClass]
    public class MvcApplicationInit : TestBase
    {
        [TestInitialize]
        public void TestCleanup()
        {
            MugenInjectionHttpApplication.ApplicationConfiguration = null;
        }

        [TestMethod]
        public void InitEmptyAppTest()
        {
            var mvc = new MvcApplication();
            TestUtils.Catch<NullReferenceException>(() => MugenInjectionHttpApplication.ApplicationConfiguration.ToString());

            using (var mugenInjector = GetInjector())
            {
                //Empty configuration
                mvc.ApplicationConfig = configuration =>
                {

                };
                mvc.InjectorFunc = () => mugenInjector;
                mvc.Application_Start();
                MugenInjectionHttpApplication.ApplicationConfiguration.ShouldNotBeNull();
                MugenInjectionHttpApplication.ApplicationConfiguration.Injector.ShouldEqual(mugenInjector);
                
                TestUtils.Catch<NotSupportedException>(() => mugenInjector.Bind<MvcApplication>().ToSelf().InSessionScope());
                TestUtils.Catch<NotSupportedException>(() => mugenInjector.Bind<MvcApplication>().ToSelf().InHttpRequestScope());
                TestUtils.Catch<NotSupportedException>(() => mugenInjector.Bind<MvcApplication>().ToSelf().InOperationRequestScope());
            }
        }

        [TestMethod]
        public void InitMvcAppTest()
        {
            var mvc = new MvcApplication();
            TestUtils.Catch<NullReferenceException>(() => MugenInjectionHttpApplication.ApplicationConfiguration.ToString());

            using (var mugenInjector = GetInjector())
            {
                IApplicationConfiguration configurationGlobal = null;
                //Empty configuration
                mvc.ApplicationConfig = configuration =>
                {
                    configurationGlobal = configuration;
                    configuration.WithHttpConfiguration().WithMvcConfiguration();
                };
                mvc.InjectorFunc = () => mugenInjector;
                mvc.Application_Start();
                MugenInjectionHttpApplication.ApplicationConfiguration.ShouldEqual(configurationGlobal);
                MugenInjectionHttpApplication.ApplicationConfiguration.Injector.ShouldEqual(mugenInjector);
                MugenInjectionHttpApplication.ApplicationConfiguration.Configurations.Count.ShouldEqual(2);

                mugenInjector.Bind<MvcApplication>().ToSelf().InSessionScope();
                mugenInjector.Bind<MvcApplication>().ToSelf().InHttpRequestScope();
                TestUtils.Catch<NotSupportedException>(() => mugenInjector.Bind<MvcApplication>().ToSelf().InOperationRequestScope());
            }            
        }
    }

    public class MvcApplication : MugenInjectionHttpApplication
    {
        #region

        public Func<IInjector> InjectorFunc { get; set; }
        public Action<IApplicationConfiguration> ApplicationConfig { get; set; }

        #endregion

        #region Overrides of MugenInjectionHttpApplication

        /// <summary>
        /// Creates the injector that will manage your application.
        /// </summary>
        /// <returns>The created injector.</returns>
        protected override IInjector CreateInjector()
        {
            return InjectorFunc();
        }

        /// <summary>
        /// Configure current <see cref="MugenInjectionHttpApplication"/>
        /// </summary>
        /// <param name="applicationConfiguration">Configuration instance.</param>
        protected override void Configure(IApplicationConfiguration applicationConfiguration)
        {
            ApplicationConfig(applicationConfiguration);
        }

        #endregion
    }
}