﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MugenInjection;
using MugenInjection.Interface;

namespace MugenInjectionTest.Core
{
    public partial class InjectorResolverTest
    {
        [TestMethod]
        public void TestGetAllInCurrentGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters);
                var allInCurrent = injector.GetAllInCurrent<string>(parameters);
                Assert.AreEqual(allInCurrent.Count(), 2);
                Assert.AreEqual(allInCurrent.ElementAt(0), string.Empty);
                Assert.AreEqual(allInCurrent.ElementAt(1), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict);
                var allInCurrent = injector.GetAllInCurrent<string>(dict);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters);
                var allInCurrent = injector.GetAllInCurrent<string>(dict, parameters);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .NamedBinding(name);
                var allInCurrent = injector.GetAllInCurrent<string>(name);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict)
                    .NamedBinding(name);
                var allInCurrent = injector.GetAllInCurrent<string>(name, dict);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters)
                    .NamedBinding(name);
                var allInCurrent = injector.GetAllInCurrent<string>(name, dict, parameters);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestGetAllInCurrentIntoGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";
            var typeInto = typeof(Attribute);

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                var allInCurrentInto = injector.GetAllInCurrentInto<string>(TypeInto, ParameterInfo, null, parameters);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                var allInCurrentInto = injector.GetAllInCurrentInto<string>(TypeInto, ParameterInfo, dict);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                var allInCurrentInto = injector.GetAllInCurrentInto<string>(TypeInto, ParameterInfo, dict, parameters);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                var allInCurrentInto = injector.GetAllInCurrentInto<string>(name, TypeInto, ParameterInfo);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                var allInCurrentInto = injector.GetAllInCurrentInto<string>(name, TypeInto, ParameterInfo, dict);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                var allInCurrentInto = injector.GetAllInCurrentInto<string>(name, TypeInto, ParameterInfo, dict, parameters);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestGetAllInCurrentNonGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";
            var bindType = typeof(string);

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters);
                var allInCurrent = injector.GetAllInCurrent(bindType, parameters);
                Assert.AreEqual(allInCurrent.Count(), 2);
                Assert.AreEqual(allInCurrent.ElementAt(0), string.Empty);
                Assert.AreEqual(allInCurrent.ElementAt(1), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict);
                var allInCurrent = injector.GetAllInCurrent(bindType, dict);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters);
                var allInCurrent = injector.GetAllInCurrent(bindType, dict, parameters);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .NamedBinding(name);
                var allInCurrent = injector.GetAllInCurrent(bindType, name);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict)
                    .NamedBinding(name);
                var allInCurrent = injector.GetAllInCurrent(bindType, name, dict);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters)
                    .NamedBinding(name);
                var allInCurrent = injector.GetAllInCurrent(bindType, name, dict, parameters);
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestGetAllInCurrentIntoNonGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";
            var typeInto = typeof(Attribute);
            var bindType = typeof(string);

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                var allInCurrentInto = injector.GetAllInCurrentInto(bindType, TypeInto, ParameterInfo, null, parameters);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                var allInCurrentInto = injector.GetAllInCurrentInto(bindType, TypeInto, ParameterInfo, dict);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                var allInCurrentInto = injector.GetAllInCurrentInto(bindType, TypeInto, ParameterInfo, dict, parameters);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                var allInCurrentInto = injector.GetAllInCurrentInto(bindType, name, TypeInto, ParameterInfo);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                var allInCurrentInto = injector.GetAllInCurrentInto(bindType, name, TypeInto, ParameterInfo, dict);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                var allInCurrentInto = injector.GetAllInCurrentInto(bindType, name, TypeInto, ParameterInfo, dict, parameters);
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestTryGetAllInCurrentGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters);
                Assert.IsTrue(injector.TryGetAllInCurrent(out allInCurrent, parameters));
                Assert.AreEqual(allInCurrent.Count(), 2);
                Assert.AreEqual(allInCurrent.ElementAt(0), string.Empty);
                Assert.AreEqual(allInCurrent.ElementAt(1), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict);
                Assert.IsTrue(injector.TryGetAllInCurrent(out allInCurrent, dict));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters);
                Assert.IsTrue(injector.TryGetAllInCurrent(out allInCurrent, dict, parameters));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .NamedBinding(name);
                Assert.IsTrue(injector.TryGetAllInCurrent(out allInCurrent, name));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict)
                    .NamedBinding(name);
                Assert.IsTrue(injector.TryGetAllInCurrent(out allInCurrent, name, dict));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters)
                    .NamedBinding(name);
                Assert.IsTrue(injector.TryGetAllInCurrent(out allInCurrent, name, dict, parameters));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestTryGetAllInCurrentNonGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";
            var bindType = typeof(string);

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters);
                Assert.IsTrue(injector.TryGetAllInCurrent(bindType, out allInCurrent, parameters));
                Assert.AreEqual(allInCurrent.Count(), 2);
                Assert.AreEqual(allInCurrent.ElementAt(0), string.Empty);
                Assert.AreEqual(allInCurrent.ElementAt(1), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict);
                Assert.IsTrue(injector.TryGetAllInCurrent(bindType, out allInCurrent, dict));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters);
                Assert.IsTrue(injector.TryGetAllInCurrent(bindType, out allInCurrent, dict, parameters));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .NamedBinding(name);
                Assert.IsTrue(injector.TryGetAllInCurrent(bindType, out allInCurrent, name));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict)
                    .NamedBinding(name);
                Assert.IsTrue(injector.TryGetAllInCurrent(bindType, out allInCurrent, name, dict));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrent;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(TypeInto.AsType());
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters)
                    .NamedBinding(name);
                Assert.IsTrue(injector.TryGetAllInCurrent(bindType, out allInCurrent, name, dict, parameters));
                Assert.AreEqual(allInCurrent.Count(), 1);
                Assert.AreEqual(allInCurrent.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestTryGetAllInCurrentIntoGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";
            var typeInto = typeof(Attribute);

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters && context.TypeInto == TypeInto.AsType() &&
                                     context.ParameterInfo == ParameterInfo);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(out allInCurrentInto, TypeInto, ParameterInfo, null, parameters));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(out allInCurrentInto, TypeInto, ParameterInfo, dict));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(out allInCurrentInto, TypeInto, ParameterInfo, dict, parameters));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(out allInCurrentInto, name, TypeInto, ParameterInfo));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(out allInCurrentInto, name, TypeInto, ParameterInfo, dict));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<string> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(out allInCurrentInto, name, TypeInto, ParameterInfo, dict,
                                                              parameters));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
        }

        [TestMethod]
        public void TestTryGetAllInCurrentIntoNonGeneric()
        {
            var parameters = new IInjectionParameter[1];
            var dict = new Dictionary<string, object>();
            const string name = "test";
            const string value = "value";
            var typeInto = typeof(Attribute);
            var bindType = typeof(string);

            //Only parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.Parameters == parameters && context.TypeInto == TypeInto.AsType() &&
                                     context.ParameterInfo == ParameterInfo);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(bindType, out allInCurrentInto, TypeInto, ParameterInfo, null, parameters));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>().ToConstant(value).When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(bindType, out allInCurrentInto, TypeInto, ParameterInfo, dict));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only special parameters and parameters
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(bindType, out allInCurrentInto, TypeInto, ParameterInfo, dict, parameters));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(bindType, out allInCurrentInto, name, TypeInto, ParameterInfo));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(bindType, out allInCurrentInto, name, TypeInto, ParameterInfo, dict));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
            //Only named binding and special parameters and parameters.
            using (MugenInjector injector = GetInjector())
            {
                IEnumerable<object> allInCurrentInto;
                injector.Bind<string>().ToConstant(string.Empty).WhenInto(typeInto);
                injector.Bind<string>()
                    .ToConstant(value)
                    .When(context => context.SpecialParameters == dict && context.Parameters == parameters && context.TypeInto == TypeInto.AsType() && context.ParameterInfo == ParameterInfo)
                    .NamedBinding(name);

                Assert.IsTrue(injector.TryGetAllInCurrentInto(bindType, out allInCurrentInto, name, TypeInto, ParameterInfo, dict,
                                                              parameters));
                Assert.AreEqual(allInCurrentInto.Count(), 1);
                Assert.AreEqual(allInCurrentInto.ElementAt(0), value);
            }
        }
    }
}
